/*
  implements a blocking file descriptor
  Copyright (C) 1998  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */


#include <util/fifo.h>



Fifo::Fifo(char* filename,int flags) {
  buffer=new Buffer(50);
  buffer->append(filename);
  this->flags=flags;
  fd=-2;
  ::mkfifo(buffer->getData(),S_IRUSR|S_IWUSR);
}


Fifo::~Fifo() {
  ::unlink(buffer->getData());
  delete buffer;
    
}

int Fifo::open() {
  fd=::open(buffer->getData(),flags);
  if (fd < 0) {
    cout << "open fifo failed !!!!!!!!!!!!!!"<<endl;
  }
  return fd;
}
  
int Fifo::close() {
  int ret=0;
  if (fd >= 0) {
    ret=::close(fd);
    fd=-2;
  }
  if (ret < 0) {
    cout << "close fifo failed !!!!!!!!!!!!!!"<<endl;
  }
  return ret;
}
 

char* Fifo::getFileName() {
  return buffer->getData();
}



int Fifo::read(char* buf, int max) {
  int back;

  if (fd < 0) {
    cout << "no filedescriptor in fifo"<<endl;
    usleep(_TIMEOUT_MS);
    return 0;
  }
  
  back=::read(fd,buf,max);
  

  if (back == -1) {
    printf("fifo error mit :%d fd:%d buf:%p len:%d\n",
	   errno,fd,buf,max);
    if (errno == EAGAIN) {
      printf("errno:EAGAIN\n");
    }
    if (errno == EINTR) {
      printf("errno:EINTR\n");
    }
    if (errno == EIO) {
      printf("errno:EIO\n");
    }
    if (errno == EBADF) {
      printf("errno:EBADF\n");
    }    
    if (errno == EINVAL) {
      printf("errno:EINVAL\n");
    }  
    if (errno == EISDIR) {
      printf("errno:EISDIR\n");
    }  
    if (errno == EFAULT) {
      printf("errno:EFAULT\n");
    } 
  }
  return back;
}
  
 
int Fifo::write(char* buf, int max) {
  return ::write(fd,buf,max);
}


int Fifo::isOpen() {
  return (fd>=0);
}
  
